/*****************************************************************************
*                                                                            *
* EaxTest Sample demo for EAX								                 *
*                                                                            *
* (C) Copyright Creative Technology Ltd. 1992-98. All rights reserved	     *
* worldwide.                                                                 *
*                                                                            *
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY      *
* KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE	     *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR      *
* PURPOSE.                                                                   *
*                                                                            *
*    You have a royalty-free right to use, modify, reproduce and distribute  *
*    the sample Files (and/or any modified version) in any way you find      *
*    useful, provided that you agree that Creative has no warranty           *
*    obligations or liability for any sample Files.                          *
*                                                                            *
******************************************************************************/
/*****************************************************************************\
*    File name   : Eaxtest.CPP                                               *
*                                                                            *
*    Programmers : Ifor Powell Creative Labs (UK), 1998. All rights reserved.*
*          		 : Other pieces of code from all sorts of sources.			 *
*                                                                            *
*    Version     : 1.0                                                       *
*                                                                            *
*                                                                            *
*    EAXTest is a sample application that uses the EAX extentions to DS3D    *
*	 It's fairly simple stuff just to show the basics of how easy it is.     *
*	 The EAX part of the code is actualy very simple.						 *
*																			 *
*    Note that the code at present takes no account of multiple apps         *
*    Working with EAX at the same time.....									 *
*                                                                            *
******************************************************************************/

#include <windows.h>
#include <mmsystem.h>
#include <commctrl.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "dsound.h"
#include "resource.h"
#include "eaxtest.h"
#include "eax.h"

/* Constants */
#define WIN_WIDTH	400
#define WIN_HEIGHT	400

/* Function declarations */
BOOL InitializeDSound( void );
void FreeDSound( void );
void DrawBitmap(HDC, HBITMAP ,int ,int ,DWORD);
BOOL FetchUserWaveFile(HWND);
BOOL OpenSecondaryBuffer(LPSTR );
BOOL TestEaxPresent( void );

// default environments to put in menu.
char*	EnvironmentName[EAX_ENVIRONMENT_COUNT] = {	
	"EAX_ENVIRONMENT_GENERIC",                
    "EAX_ENVIRONMENT_PADDEDCELL",
    "EAX_ENVIRONMENT_ROOM",              
    "EAX_ENVIRONMENT_BATHROOM",
    "EAX_ENVIRONMENT_LIVINGROOM",
    "EAX_ENVIRONMENT_STONEROOM",
    "EAX_ENVIRONMENT_AUDITORIUM",
    "EAX_ENVIRONMENT_CONCERTHALL",
    "EAX_ENVIRONMENT_CAVE",
    "EAX_ENVIRONMENT_ARENA",
    "EAX_ENVIRONMENT_HANGAR",
    "EAX_ENVIRONMENT_CARPETEDHALLWAY",
    "EAX_ENVIRONMENT_HALLWAY",
    "EAX_ENVIRONMENT_STONECORRIDOR",
    "EAX_ENVIRONMENT_ALLEY",
    "EAX_ENVIRONMENT_FOREST",
    "EAX_ENVIRONMENT_CITY",
    "EAX_ENVIRONMENT_MOUNTAINS",
    "EAX_ENVIRONMENT_QUARRY",
    "EAX_ENVIRONMENT_PLAIN",
    "EAX_ENVIRONMENT_PARKINGLOT",
    "EAX_ENVIRONMENT_SEWERPIPE",
    "EAX_ENVIRONMENT_UNDERWATER",
    "EAX_ENVIRONMENT_DRUGGED",
    "EAX_ENVIRONMENT_DIZZY",
    "EAX_ENVIRONMENT_PSYCHOTIC"
	};

unsigned long	EnvironmentValue[EAX_ENVIRONMENT_COUNT] = {
	EAX_ENVIRONMENT_GENERIC, 
    EAX_ENVIRONMENT_PADDEDCELL,
    EAX_ENVIRONMENT_ROOM,
    EAX_ENVIRONMENT_BATHROOM,
    EAX_ENVIRONMENT_LIVINGROOM,
    EAX_ENVIRONMENT_STONEROOM,
    EAX_ENVIRONMENT_AUDITORIUM,
    EAX_ENVIRONMENT_CONCERTHALL,
    EAX_ENVIRONMENT_CAVE,
    EAX_ENVIRONMENT_ARENA,
    EAX_ENVIRONMENT_HANGAR,
    EAX_ENVIRONMENT_CARPETEDHALLWAY,
    EAX_ENVIRONMENT_HALLWAY,
    EAX_ENVIRONMENT_STONECORRIDOR,
    EAX_ENVIRONMENT_ALLEY,
    EAX_ENVIRONMENT_FOREST,
    EAX_ENVIRONMENT_CITY,
    EAX_ENVIRONMENT_MOUNTAINS,
    EAX_ENVIRONMENT_QUARRY,
    EAX_ENVIRONMENT_PLAIN,
    EAX_ENVIRONMENT_PARKINGLOT,
    EAX_ENVIRONMENT_SEWERPIPE,
    EAX_ENVIRONMENT_UNDERWATER,
    EAX_ENVIRONMENT_DRUGGED,
    EAX_ENVIRONMENT_DIZZY,
    EAX_ENVIRONMENT_PSYCHOTIC
	};


/* Application instance and handle */
HANDLE          hInst;
HWND            hWndDemo        = 0;

/* Direct Sound objects */
/* Simple demo only ever has one secondary buffer which is global */

LPDIRECTSOUND lpDirectSound = NULL;				//Direct sound
LPDIRECTSOUNDBUFFER lpdsbPrimary = NULL;		//Primary buffer
LPDIRECTSOUND3DLISTENER lp3DListener = NULL;	//Listener
LPDIRECTSOUNDBUFFER lpSecondaryBuffer = NULL;	//Secondary Buffer
LPDIRECTSOUND3DBUFFER lpDs3dBuffer = NULL;		//3d interface for secondary buffer
LPKSPROPERTYSET lpPropertySet = NULL;			//Property set interface for secondary buffer

/* Global structure in which we track the listener and the secondary buffer. positions. */

DS3DBUFFER Buffer3dProps = {sizeof(DS3DBUFFER),		// size
						0,0,0,						// Position  Gets updated as things move.
						0,0,0,						// Velocity  Fixed for this demo.
						0,							// Inner angle
						0,							// Outer angle
						1,0,0,						// Cone orientation
						0,							// Cone outer volume
						DS3D_DEFAULTMINDISTANCE,	// Min distance
						DS3D_DEFAULTMAXDISTANCE,	// Max distance
						DS3DMODE_NORMAL				// Mode
						};

DS3DLISTENER Listener3dProps = {
						sizeof(DS3DLISTENER),		// Size
						0,0,0,						// Position   Gets set to center of screen coordinates
						0,0,0,						// Velicity   Fixed for this demo
						0,0,1,						// Orientation Front
						0,1,0,						// Orientation Top
						DS3D_DEFAULTDISTANCEFACTOR,	// Distance Factor
						DS3D_DEFAULTROLLOFFFACTOR,	// Rolloff factor
						DS3D_DEFAULTDOPPLERFACTOR	// Doppler factor
						};

float			fReverbMix = EAX_REVERBMIX_USEDISTANCE;	//Curent reverb mix setting
float			fScaleFactor = 0.1f;				// Used for converting mouse cords to DS3D Cords.
char		    szUserFileName[128];				// Wave file name the user has chosen to open
BOOL			bDrag = FALSE;						// Controlls whether the emiter follows the mouse.
int				cx = WIN_WIDTH / 2;					// Center Screen Coordinates for listener
int				cy = WIN_HEIGHT / 2;
int				x_Source = 0;						// Position of sound
int				y_Source = 0;
HBRUSH 			hbr_Green;
HBITMAP			hBmpMan;
HBITMAP 		hBmpChopper;

// Portable Macros for splitting WM_COMMAND
#define GET_WM_COMMAND_CMD(wp, lp) HIWORD(wp)
#define GET_WM_COMMAND_ID(wp, lp) LOWORD(wp)
#define GET_WM_COMMAND_HWND(wp, lp) (HWND) (lp)

int APIENTRY WinMain(HINSTANCE	hInstance, 
										 HINSTANCE	hPrevInstance, 
										 LPSTR		lpCmdLine, 
										 int		nCmdShow)
                     
{
	MSG 	msg;
	hbr_Green = CreateSolidBrush(RGB(0,255,0));

	if (hPrevInstance)
		return FALSE;
	else
	if (!InitApplication(hInstance))
		return (FALSE);

	if (!InitInstance(hInstance, nCmdShow))
		return (FALSE);
	
	while (GetMessage(&msg, NULL, 0, 0) )
		{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
		}
	DeleteObject(hbr_Green);
	return (msg.wParam);
}


BOOL InitApplication(HANDLE hInstance)
{
	WNDCLASS  wc;

	wc.style = 0;
	wc.lpfnWndProc = MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(hInstance, (LPCSTR)IDI_ICON1);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = hbr_Green;
	wc.lpszMenuName =  "EAXTestMenu";
	wc.lpszClassName = "EAXTestClass";

	return (RegisterClass(&wc));
}


BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{
	HWND	    hWnd;
	HMENU		hmenu, hNewMenu;
	int			i;

	hInst = hInstance;
	hWnd = CreateWindow("EAXTestClass",
											"EAX 3D Audio Sample Demo",
											WS_OVERLAPPEDWINDOW, 
											CW_USEDEFAULT,
											CW_USEDEFAULT,
											WIN_WIDTH,
											WIN_HEIGHT,
											NULL,
											NULL,
											hInstance,
											NULL);

	if (!hWnd)
		return (FALSE);
	
	hWndDemo = hWnd;

	if (!InitializeDSound()) return (FALSE);
	if (!TestEaxPresent())
	{
		MessageBox( hWndDemo, "Eax extentions are not present",
				"No Eax", MB_OK | MB_ICONSTOP );
	}
	else
	{
		// We have EAX so add the envoronment menu.
		hmenu = GetMenu(hWndDemo);
		if (GetMenuItemCount(hmenu) <3)
		{
			// make a popupmenu
			hNewMenu = CreatePopupMenu();
			for (i = 0; i < EAX_ENVIRONMENT_COUNT; i++) 
			{
				//add in all the envoronments.
				AppendMenu(hNewMenu, MF_STRING,
					IMD_ENVIRONMENT_MENU + EnvironmentValue[i], EnvironmentName[i]);
			}
			// And add the popup to the main menu at the end.
			AppendMenu(hmenu, MF_POPUP, (int) hNewMenu, "Environments");
			DrawMenuBar(hWndDemo);
		}
	}
	ShowWindow(hWnd, nCmdShow);
	return (TRUE);
}



long APIENTRY  MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC				hdc;
	PAINTSTRUCT		ps;
	RECT			Rect;
	HRESULT			hr;
	ULONG			ulEnvironment;
	ULONG			ulSize;
	HMENU			hmenu;
	


	switch (message)
		{
		case WM_CREATE:
			/* Load all bitmaps */
			
			hBmpMan = LoadBitmap(hInst, "man_xz");		//Man represents listener
			hBmpChopper = LoadBitmap(hInst, "heli_xz");	// chopper represents emmiter.
			return(0);

		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			DrawBitmap(hdc, hBmpMan, cx-32, cy-32,  SRCCOPY);	//Listener at center of screen
			DrawBitmap(hdc, hBmpChopper, x_Source-16, y_Source-16, SRCINVERT);	// emiter at stored x,y
			EndPaint(hWnd, &ps);
			return(0);


		case WM_SIZE:
			// calculate new center coords
			GetClientRect( hWnd, &Rect);
			cx = Rect.right/2;
			cy = Rect.bottom/2;
			InvalidateRect(hWnd, NULL, TRUE);

			// Update the listener position
			Listener3dProps.vPosition.x = cx * fScaleFactor;
			Listener3dProps.vPosition.z = cy * -fScaleFactor;
			// If we have Created the listener then move the position
			if (lp3DListener != NULL)
			{
				if( FAILED( hr = lp3DListener->SetAllParameters(&Listener3dProps,DS3D_IMMEDIATE)))
				{
					MessageBox( hWndDemo, "Couldn't setAllParameters for 3D listener",
						"SetAllParams Error", MB_OK | MB_ICONSTOP );
				}
			}
			return(0);

		case WM_MOUSEMOVE:
			// if bDrag then use mouse position for emiter position
			if(bDrag) 
			{
				hdc = GetDC(hWnd);
				DrawBitmap(hdc, hBmpChopper, x_Source-16, y_Source-16, SRCINVERT);
				x_Source = LOWORD(lParam);
				y_Source = HIWORD(lParam);
				DrawBitmap(hdc, hBmpChopper, x_Source-16, y_Source-16, SRCINVERT);
				ReleaseDC(hWnd, hdc);
				// update the buffer position.
				Buffer3dProps.vPosition.x = x_Source * fScaleFactor;
				Buffer3dProps.vPosition.z = y_Source * -fScaleFactor;
				// If we have a 3d interface then call it to do the actual move.
				if(lpDs3dBuffer != NULL)
				{
					if (FAILED(	hr = lpDs3dBuffer->SetAllParameters(&Buffer3dProps, DS3D_IMMEDIATE))){
						MessageBox( hWndDemo, "Could not SetAllParams() for DS3D buffer",
							"SetAllParams Error",MB_OK | MB_ICONSTOP );
					}
				}
			}
			return(0);

		case WM_LBUTTONDOWN:
			// Ctrl and left click togles drag mode on and off.  
			// normal left click just moves emiter to mouse position.
			if(wParam & MK_CONTROL) {
				bDrag = !bDrag;
			}
			hdc = GetDC(hWnd);
			DrawBitmap(hdc, hBmpChopper, x_Source-16, y_Source-16, SRCINVERT);
			x_Source = LOWORD(lParam);
			y_Source = HIWORD(lParam);
			DrawBitmap(hdc, hBmpChopper, x_Source-16, y_Source-16, SRCINVERT);
			ReleaseDC(hWnd, hdc);
			// update the position
			Buffer3dProps.vPosition.x = x_Source * fScaleFactor;
			Buffer3dProps.vPosition.z = y_Source * -fScaleFactor;
			// and if we have the interface do the move.
			if(lpDs3dBuffer != NULL)
			{
				if (FAILED(hr = lpDs3dBuffer->SetAllParameters(&Buffer3dProps, DS3D_IMMEDIATE))){
					MessageBox( hWndDemo, "Could not SetAllParams() for DS3D buffer",
							"SetAllParams Error",MB_OK | MB_ICONSTOP );
				}
			}
			return(0);	

		case WM_RBUTTONDOWN:
			// Right click just restarts sound if we have one loaded.
			if(lpSecondaryBuffer != NULL)
			{
				if (FAILED(hr = lpSecondaryBuffer->SetCurrentPosition(0)))
				{
					MessageBox( hWndDemo, "Could not reset curent position to start",
						"SetCurrentPosition Failed", MB_OK | MB_ICONSTOP );
				}
				// If the buffer is not playing then we need to start it.
				// If it's already playing this call just gets ignored.
				if (FAILED(hr = lpSecondaryBuffer->Play( 0, 0, DSBPLAY_LOOPING )))
				{
					MessageBox( hWndDemo, "Could not Play() secondary buffer",
						"Paly Failed", MB_OK | MB_ICONSTOP );
				}
			}
			return(0);	

		case WM_COMMAND:
			{
			WORD Cmd = GET_WM_COMMAND_CMD (wParam, lParam) ;
			WORD ID  = GET_WM_COMMAND_ID  (wParam, lParam) ;
			HWND hWndCtrl =  GET_WM_COMMAND_HWND ( wparam, lParam) ;

			if ((ID >= IMD_ENVIRONMENT_MENU) & (ID <= IMD_ENVIRONMENT_MENU + EAX_ENVIRONMENT_COUNT))
			{
				// we are choosing an environment
				if(lpPropertySet == NULL)
				{
					// don't have property set yet so get it.
					if (!TestEaxPresent())
					{
						// Should not realy get here as if eax is not present the menu should not be made.
						MessageBox( hWndDemo, "Eax extentions are not present.\nSecondary buffer in software?",
								"No Eax", MB_OK | MB_ICONSTOP );
					}
				}
				if(lpPropertySet != NULL)
				{
					// we now have a valid property set so start by getting the old Environment and clearing it's menu check.
					if (FAILED(hr = lpPropertySet->Get(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_ENVIRONMENT,
						NULL,0,&ulEnvironment,sizeof(unsigned long),&ulSize)))
					{
						MessageBox( hWndDemo, "Could not Get Environment",
							"GetEnvironment failed",  MB_OK | MB_ICONSTOP );
					}
					else
					{
						hmenu = GetSubMenu(GetMenu(hWndDemo), 2);
						// look to uncheck the old menu.
						CheckMenuItem(hmenu, ulEnvironment + IMD_ENVIRONMENT_MENU, MF_BYCOMMAND | MF_UNCHECKED);
					}
					// Now set the new Environment
					ulEnvironment = ID - IMD_ENVIRONMENT_MENU;
					if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_ENVIRONMENT,
						NULL,0,&ulEnvironment,sizeof(unsigned long))))
					{
						MessageBox( hWndDemo, "Could not Set Environment",
							"SetEnvironment failed", MB_OK | MB_ICONSTOP );
					}
					else
					{
						hmenu = GetSubMenu(GetMenu(hWndDemo), 2);
						// look to check the new menu.
						CheckMenuItem(hmenu, ulEnvironment + IMD_ENVIRONMENT_MENU, MF_BYCOMMAND | MF_CHECKED);
					}

				}

			}
			else
			{
				// It is a standard menu item.
				switch (ID)
					{
					case IDM_FILE_OPEN:
						// Get the file name to try and load
						if ( FetchUserWaveFile(hWnd) )
						{
							// first stop and remove any buffer we are curently playing.
							if(lpSecondaryBuffer != NULL)
							{
								if (FAILED(hr = lpSecondaryBuffer->Stop()))
								{
									MessageBox( hWndDemo, "Could not Stop() secondary buffer",
										"Stop Failed", MB_OK | MB_ICONSTOP );
								}
								// if there is a 3dbuffer interface then release it.
									if ( NULL != lpDs3dBuffer)
									{
										if (FAILED(hr = lpDs3dBuffer->Release()))
										{
											MessageBox( hWndDemo, "Could not Release() 3d buffer interface",
															"Release Failed", MB_OK | MB_ICONSTOP );
										}
										lpDs3dBuffer = NULL;
									}
								// and release the secondary
								if (FAILED(hr = lpSecondaryBuffer->Release()))
								{
									MessageBox( hWndDemo, "Could not Release() secondary buffer",
										"Release Failed",MB_OK | MB_ICONSTOP );
								}
								lpSecondaryBuffer = NULL;
							}
							// then open the new secondary buffer.
							if (OpenSecondaryBuffer(szUserFileName)) 
							{	
								// The reverbmix is automatic on a new buffer.
								fReverbMix = EAX_REVERBMIX_USEDISTANCE;
								// Get the new buffer playing
								if (FAILED(hr = lpSecondaryBuffer->Play( 0, 0, DSBPLAY_LOOPING )))
								{
									MessageBox( hWndDemo, "Could not Play() secondary buffer",
										"Paly Failed", MB_OK | MB_ICONSTOP );
								}
							}
						}
						InvalidateRect(hWnd, NULL, TRUE);  
						break;
					case IDM_FILE_STOP:
						// just Stop the sound if we have one loaded.
						if(lpSecondaryBuffer != NULL)
						{
							if (FAILED(hr = lpSecondaryBuffer->Stop()))
							{
								MessageBox( hWndDemo, "Could not Stop ",
									"Stop Failed", MB_OK | MB_ICONSTOP );
							}
						}
						break;
					case IDM_EAX_TWEAK:
						// bring up the dialog box for tweaking the parameters.
						DialogBox(hInst,
											"TWEAK_DIALOG",
											hWnd,
											(DLGPROC) Tweak);
						break;
					case ID_QUIT:
						PostMessage(hWnd, WM_DESTROY, 0, 0);
						break;
					case IDM_ABOUT:
						DialogBox(hInst,
											"AboutBox",
											hWnd,
											(DLGPROC) About);
						break;
					} // end switch(ID)
				}  //end else.
			return(0);
			} // end WM_COMMAND


		case WM_DESTROY:
			FreeDSound();
			PostQuitMessage(0);
			return(0);
	} // end switch(message)
	return (DefWindowProc(hWnd, message, wParam, lParam));
}




LRESULT CALLBACK About(HWND	hDlg, UINT  message, WPARAM	wParam, LPARAM	lParam)
{
	switch (message)
		{
		case WM_INITDIALOG:
			return (TRUE);

		case WM_COMMAND:
			if (wParam == IDOK || wParam == IDCANCEL)
				{
				EndDialog(hDlg, TRUE);
				return (TRUE);
				}
			break;
		}
	return (FALSE);
}


/* Deal with the tweak dialog box which has sliders for Reverb Volume, DecayTime, and dampling.
 * Which are all assosiated with the listener
 * There is also a slider for reverb mix which is assosiated with the buffer
 * A new buffer will always have this under audomatic controll.
 */
LRESULT CALLBACK Tweak(HWND	hDlg, UINT  message, WPARAM	wParam, LPARAM	lParam)
{
static HWND    hVolumeSlider, hVolumeText;
static HWND    hDecayTimeSlider, hDecayTimeText;
static HWND    hDampingSlider, hDampingText;
static HWND    hReverbMixSlider, hReverbMixText, hReverbMixCheck;

HRESULT		hr;
int			iSliderValue;
float		fVolume;
float		fDecayTime;
float		fDamping;
ULONG		ulSize;
char		szText[8];

	switch (message)
		{
		case WM_INITDIALOG:
			if (!TestEaxPresent())
			{
				MessageBox( hWndDemo, "No EAX so no Tweaking it",
							"No Eax", MB_OK | MB_ICONSTOP );
				// No EAX so quit straight out.
				DestroyWindow( hDlg );
			}
			else
			{
				//set up the Volume slider corectly 0.0 - 1.0 map to 0 - 100
				hVolumeSlider = GetDlgItem( hDlg, IDC_SLIDER_VOLUME );
				hVolumeText = GetDlgItem( hDlg, IDC_VOLUME_TEXT );
				SendMessage( hVolumeSlider, TBM_SETRANGEMIN, FALSE, 0);
				SendMessage( hVolumeSlider, TBM_SETRANGEMAX, FALSE, 100);
				SendMessage( hVolumeSlider, TBM_SETPAGESIZE, 0, 1);
				//get the curent volume and set the slider to it.
				if (FAILED(hr = lpPropertySet->Get(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_VOLUME,
						NULL,0,&fVolume,sizeof(unsigned long),&ulSize)))
				{
					MessageBox( hWndDemo, "Get Volume failed",
							"Could not Get Volume", MB_OK | MB_ICONSTOP );
				}
				else
				{
					sprintf( szText, "%.2f", fVolume );
					SendMessage( hVolumeText, WM_SETTEXT, 0, (LPARAM) szText);
					SendMessage( hVolumeSlider, TBM_SETPOS, TRUE, (int) (fVolume * 100));
				}
				
				//set up the DecayTime slider corectly 0.1 to 20.0 map to 1 to 200
				hDecayTimeSlider = GetDlgItem( hDlg, IDC_SLIDER_DECAYTIME );
				hDecayTimeText = GetDlgItem( hDlg, IDC_DECAYTIME_TEXT );
				SendMessage( hDecayTimeSlider, TBM_SETRANGEMIN, FALSE, 1);
				SendMessage( hDecayTimeSlider, TBM_SETRANGEMAX, FALSE, 200);
				SendMessage( hDecayTimeSlider, TBM_SETPAGESIZE, 0, 1);
				//get the curent DecayTime and set the slider to it.
				if (FAILED(hr = lpPropertySet->Get(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_DECAYTIME,
						NULL,0,&fDecayTime,sizeof(unsigned long),&ulSize)))
				{
					MessageBox( hWndDemo, "Get DecayTime failed",
							"Could not Get DecayTime", MB_OK | MB_ICONSTOP );
				}
				else
				{
					sprintf( szText, "%.1f", fDecayTime );
					SendMessage( hDecayTimeText, WM_SETTEXT, 0, (LPARAM) szText);
					SendMessage( hDecayTimeSlider, TBM_SETPOS, TRUE, (int) (fDecayTime * 10));
				}

				//set up the Damping slider corectly 0.0 to 2.0 map to 0 to 200
				hDampingSlider = GetDlgItem( hDlg, IDC_SLIDER_DAMPING );
				hDampingText = GetDlgItem( hDlg, IDC_DAMPING_TEXT );
				SendMessage( hDampingSlider, TBM_SETRANGEMIN, FALSE, 0);
				SendMessage( hDampingSlider, TBM_SETRANGEMAX, FALSE, 200);
				SendMessage( hDampingSlider, TBM_SETPAGESIZE, 0, 1);
				//get the curent Damping and set the slider to it.
				if (FAILED(hr = lpPropertySet->Get(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_DAMPING,
						NULL,0,&fDamping,sizeof(unsigned long),&ulSize)))
				{
					MessageBox( hWndDemo, "Get Damping failed",
							"Could not Get Damping", MB_OK | MB_ICONSTOP );
				}
				else
				{
					sprintf( szText, "%.2f", fDamping );
					SendMessage( hDampingText, WM_SETTEXT, 0, (LPARAM) szText);
					SendMessage( hDampingSlider, TBM_SETPOS, TRUE, (int) (fDamping * 100));
				}

				
				//set up the ReverbMix slider and checkbox  0.0 - 1.0 map to 0 - 100
				hReverbMixSlider = GetDlgItem( hDlg, IDC_SLIDER_REVERBMIX );
				hReverbMixText = GetDlgItem( hDlg, IDC_REVERBMIX_TEXT );
				hReverbMixCheck = GetDlgItem( hDlg, IDC_CHECK_REVERBMIX );
				SendMessage( hReverbMixSlider, TBM_SETRANGEMIN, FALSE, 0);
				SendMessage( hReverbMixSlider, TBM_SETRANGEMAX, FALSE, 100);
				SendMessage( hReverbMixSlider, TBM_SETPAGESIZE, 0, 1);
				//can not get the curent setting so have a global variable (fReverbMix) to track it. .
				if (fReverbMix == EAX_REVERBMIX_USEDISTANCE)
				{
					// Disable slider and set text to "" uncheck check box.
					sprintf( szText, "Auto");
					SendMessage( hReverbMixText, WM_SETTEXT, 0, (LPARAM) szText);
					SendMessage( hReverbMixSlider, WM_ENABLE, FALSE, 0);
					SendMessage( hReverbMixCheck, BM_SETCHECK, FALSE, 0);
				}
				else
				{
					// Enable the slider set the text and check the checkbox.
					sprintf( szText, "%.2f", fReverbMix );
					SendMessage( hReverbMixText, WM_SETTEXT, 0, (LPARAM) szText);
					SendMessage( hReverbMixSlider, TBM_SETPOS, TRUE, (int) (fReverbMix * 100));
					SendMessage( hReverbMixSlider, WM_ENABLE, TRUE, 0);
					SendMessage( hReverbMixCheck, BM_SETCHECK, TRUE, 0);
				}
			}
			return (TRUE);

		case WM_HSCROLL:
			// Somthing is happening to one of the sliders
			switch( LOWORD(wParam) )
				{
				case TB_THUMBTRACK:
				case TB_ENDTRACK:
				case TB_LINEDOWN:
				case TB_LINEUP:
				case TB_PAGEDOWN:
				case TB_PAGEUP:
					// Get the curent slider value.
					iSliderValue = SendMessage( (HWND) lParam, TBM_GETPOS, 0, 0 );
					break;
				default:
					return (FALSE);
				}
			if((HWND) lParam == hVolumeSlider)
			{
				// It's the volume slider so adjust it as nessasary.
				fVolume = ((float)iSliderValue) / 100;
				sprintf( szText, "%.2f", fVolume);
				SendMessage( hVolumeText, WM_SETTEXT, 0, (LPARAM) szText);
				if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_VOLUME,
					NULL,0,&fVolume,sizeof(float))))
				{
					MessageBox( hWndDemo, "Set Volume failed",
							"Could not Set Volume", MB_OK | MB_ICONSTOP );
				}
			}
			else 
				if((HWND) lParam == hDecayTimeSlider)
				{
					// It's the DecayTime slider so adjust it.
					fDecayTime = ((float)iSliderValue) / 10;
					sprintf( szText, "%.1f", fDecayTime);
					SendMessage( hDecayTimeText, WM_SETTEXT, 0, (LPARAM) szText);
					if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_DECAYTIME,
						NULL,0,&fDecayTime,sizeof(float))))
					{
						MessageBox( hWndDemo, "Set DecayTime failed",
								"Could not Set DecayTime", MB_OK | MB_ICONSTOP );
					}
				}
				else
					if((HWND) lParam == hDampingSlider)
					{
						// And the same for the damping slider.
						fDamping = ((float)iSliderValue) / 100;
						sprintf( szText, "%.2f", fDamping);
						SendMessage( hDampingText, WM_SETTEXT, 0, (LPARAM) szText);
						if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAX_ReverbProperties, DSPROPERTY_EAX_DAMPING,
							NULL,0,&fDamping,sizeof(float))))
						{
							MessageBox( hWndDemo, "Set Damping failed",
									"Could not Set Damping", MB_OK | MB_ICONSTOP );
						}
					}
					else
						if((HWND) lParam == hReverbMixSlider)
						{	
							// Note that for the reverb mix slider we use the EAXBUFFER property set
							// whereas for the other sliders we use the EAX property set, they affect
							// emiters this one just affects this listener.
							fReverbMix = ((float)iSliderValue) / 100;
							sprintf( szText, "%.2f", fReverbMix);
							SendMessage( hReverbMixText, WM_SETTEXT, 0, (LPARAM) szText);
							if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAXBUFFER_ReverbProperties, DSPROPERTY_EAXBUFFER_REVERBMIX,
								NULL,0,&fReverbMix,sizeof(float))))
							{
								MessageBox( hWndDemo, "Set ReverbMix failed",
										"Could not Set ReverbMix", MB_OK | MB_ICONSTOP );
							}
						}
			return (TRUE);
		case WM_COMMAND:

			if (wParam == IDOK || wParam == IDCANCEL)
				{
				EndDialog(hDlg, TRUE);
				return (TRUE);
				}
			if ((HWND) lParam == hReverbMixCheck)
			{
				// we are doing somthing to the checkbox so get it's value and update stuff.
				if (SendMessage( hReverbMixCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					// The checkbox is checked.
					// The slider needs enableing and the reverb mix needs setting.
					SendMessage( hReverbMixSlider, WM_ENABLE, TRUE, 0);
					iSliderValue = SendMessage( hReverbMixSlider, TBM_GETPOS, 0, 0 );
					fReverbMix = ((float)iSliderValue) / 100;
					sprintf( szText, "%.2f", fReverbMix);
					SendMessage( hReverbMixText, WM_SETTEXT, 0, (LPARAM) szText);
					if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAXBUFFER_ReverbProperties, DSPROPERTY_EAXBUFFER_REVERBMIX,
						NULL,0,&fReverbMix,sizeof(float))))
					{
						MessageBox( hWndDemo, "Set ReverbMix failed",
								"Could not Set ReverbMix", MB_OK | MB_ICONSTOP );
					}
				}
				else
				{
					// The slider is now not active disable it and set the reverbmix back to auto.
					SendMessage( hReverbMixSlider, WM_ENABLE, FALSE, 0);
					fReverbMix = EAX_REVERBMIX_USEDISTANCE;
					sprintf( szText, "Auto", fReverbMix);
					SendMessage( hReverbMixText, WM_SETTEXT, 0, (LPARAM) szText);
					if (FAILED(hr = lpPropertySet->Set(DSPROPSETID_EAXBUFFER_ReverbProperties, DSPROPERTY_EAXBUFFER_REVERBMIX,
						NULL,0,&fReverbMix,sizeof(float))))
					{
						MessageBox( hWndDemo, "Set ReverbMix failed",
								"Could not Set ReverbMix", MB_OK | MB_ICONSTOP );
					}
				}
				return (TRUE);
			}
			break;
		}
	return (FALSE);
}

/* InitializeDSound()
 *
 * Initialize the DirectSound object and other stuff we'll use like the
 * primary buffer and 3D listener object.  Nothing special for EAX here.
 *
 */
BOOL InitializeDSound( void )
{
    HRESULT			hr;
    DSBUFFERDESC    dsbd;
	WAVEFORMATEX	wfxPrimary;

	// Create DS
    if (FAILED(hr = DirectSoundCreate(NULL, &lpDirectSound, NULL)))
	{
		MessageBox( hWndDemo, "Unable to get a DirectSound object",
						"DirectSoundCreate Failure", MB_OK | MB_ICONSTOP );
		goto ID_ExitDSCreateError;
	}

	// Set cooperative level to Priority so we can set primary format
    if (FAILED(hr = lpDirectSound->SetCooperativeLevel(hWndDemo, DSSCL_EXCLUSIVE)))
	{
		MessageBox( hWndDemo, "Could not SetCooperativeLevel()",
			"Set CooperativeLeve Error", MB_OK | MB_ICONSTOP );
		goto ID_ExitSetCoopError;
	}

	// Now go and create the primarary buffer.
	ZeroMemory( &dsbd, sizeof(DSBUFFERDESC));
    dsbd.dwSize = sizeof(DSBUFFERDESC);
    dsbd.dwFlags = DSBCAPS_CTRL3D | DSBCAPS_PRIMARYBUFFER;

    if( FAILED( hr = lpDirectSound->CreateSoundBuffer( &dsbd, &lpdsbPrimary, NULL )))
	{
		MessageBox( hWndDemo, "Could not Create Primary Buffer()",
						"Create Primary failure", MB_OK | MB_ICONSTOP );
		goto ID_ExitCreatePrimaryError;
	}
	// We just go for max quality here but if you have non 3d stuff at 22kHz 16 bit
	// then you may want to use 22kHz 16 bit for your primary.
    ZeroMemory( &wfxPrimary, sizeof(WAVEFORMATEX));
    wfxPrimary.wFormatTag = WAVE_FORMAT_PCM;
	wfxPrimary.nChannels = 2;
	wfxPrimary.nSamplesPerSec = 44100;
	wfxPrimary.nAvgBytesPerSec = 176400;
	wfxPrimary.nBlockAlign = 4;
	wfxPrimary.wBitsPerSample = 16;
	wfxPrimary.cbSize = 0;
	if( FAILED( hr = lpdsbPrimary->SetFormat( &wfxPrimary )))
	{
		MessageBox( hWndDemo, "Couldn't SetFormat on primary buffer",
				    "SetFormat Error", MB_OK | MB_ICONSTOP );
		// If we fail with our format we just go with the default rather than bombing out.
		// It may sound bad but we should at least here somthing.
	}

	// Now get the listener interface
    if( FAILED( hr = lpdsbPrimary->QueryInterface( IID_IDirectSound3DListener,(void**)&lp3DListener)))
	{
		MessageBox( hWndDemo, "Couldn't QI primary buffer for 3D listener interface",
						"Get Listener Error", MB_OK | MB_ICONSTOP );
		goto ID_ExitGetListenerError;
	}
	// Set the listener propertys, these are all default values with the position
	// set for the middle of the window (in WM_SIZE)
	if( FAILED( hr = lp3DListener->SetAllParameters(&Listener3dProps,DS3D_IMMEDIATE))){
		MessageBox( hWndDemo, "Couldn't setAllParameters for 3D listener",
			"SetAllParams Error", MB_OK | MB_ICONSTOP );
		goto ID_ExitSetListenerError;
	}
	// Get the primary playing to avoid all that stop/start workload.
    if( FAILED( hr = lpdsbPrimary->Play(  0, 0, DSBPLAY_LOOPING )))
	{
		MessageBox( hWndDemo, "Play Primary failed",
					"Play primary failed", MB_OK | MB_ICONSTOP );
		goto ID_ExitPlayPrimaryError;
	}
    // everything initalised and we have things set up.
    return TRUE;

	// various entry points for tidy up work
ID_ExitPlayPrimaryError:
ID_ExitSetListenerError:
	if ( NULL != lp3DListener)
	{
		lp3DListener->Release();
		lp3DListener = NULL;
	}
ID_ExitGetListenerError:
	if ( NULL != lpdsbPrimary)
	{
		lpdsbPrimary->Stop();
		lpdsbPrimary->Release();
		lpdsbPrimary = NULL;
	}
ID_ExitCreatePrimaryError:
ID_ExitSetCoopError:
    if( NULL != lpDirectSound )
	{
		lpDirectSound->Release();
	}
ID_ExitDSCreateError:
	lpDirectSound = NULL;
    return FALSE;
}



/*
 * Frees up and DS stuff we may have open in the apropriate order.
 */
void FreeDSound()
{

 	if(lpPropertySet != NULL)
	{
		lpPropertySet->Release();
		lpPropertySet = NULL;
	}

	if ( NULL != lpDs3dBuffer)
	{
		lpDs3dBuffer->Release();
		lpDs3dBuffer = NULL;
	}

	if ( NULL != lpSecondaryBuffer)
	{
		lpSecondaryBuffer->Stop();
		lpSecondaryBuffer->Release();
		lpSecondaryBuffer = NULL;
	}

	if ( NULL != lp3DListener)
	{
		lp3DListener->Release();
		lp3DListener  = NULL;
	}

	if ( NULL != lpdsbPrimary)
	{
		lpdsbPrimary->Stop();
		lpdsbPrimary->Release();
		lpdsbPrimary = NULL;
	}

    if( NULL != lpDirectSound )
	{
		lpDirectSound->Release();
		lpDirectSound = NULL;
	}
}



/*
 *  TestEaxPresent()
 *	Returns true if eax is avalible false if not. 
 *  If there is no secondary buffer already one is created to get the property set interface off
 *  If everything is ok the property set interface is ready for use after this call.
 */

BOOL TestEaxPresent( void )
{

	BOOL			bEaxOK = FALSE;		// assume it's not there till we check it is.
	WAVEFORMATEX	pcmOut;				// Format of the wave for secondary buffer if we need to make one.
	DSBUFFERDESC    dsbdSecondary;		// description for creating secondary buffer if we need to make one.
    HRESULT			hr;
	ULONG			ulSupport = 0;

	if(NULL == lpSecondaryBuffer)
	{
		// we don't have a secondary to work with so we will create one.
		ZeroMemory( &dsbdSecondary, sizeof(DSBUFFERDESC));
		ZeroMemory( &pcmOut, sizeof(WAVEFORMATEX));
		// any format should do I just say 11kHz 16 bit mono
		pcmOut.wFormatTag = WAVE_FORMAT_PCM;
		pcmOut.nChannels = 1;
		pcmOut.nSamplesPerSec = 11025;
		pcmOut.nAvgBytesPerSec = 22050;
		pcmOut.nBlockAlign = 2;
		pcmOut.wBitsPerSample = 16;
		pcmOut.cbSize = 0;
		// size is just arbitary but not too small as I have seen problems with single sample buffers...
		dsbdSecondary.dwSize = sizeof(DSBUFFERDESC);
		dsbdSecondary.dwBufferBytes = 1024;
		dsbdSecondary.dwFlags = DSBCAPS_CTRL3D | DSBCAPS_STATIC ;
		dsbdSecondary.lpwfxFormat = &pcmOut;
		if( FAILED( hr = lpDirectSound->CreateSoundBuffer( &dsbdSecondary, &lpSecondaryBuffer, NULL )))
		{
			MessageBox( hWndDemo, "Create secondary for getting property set extention failed",
					"Create secondary failed", MB_OK | MB_ICONSTOP );
			// if we can't make a secondary we can't do anything else so just exit out.
			return (FALSE);
		}
		fReverbMix = EAX_REVERBMIX_USEDISTANCE;
	}

	// if there is already a property set interface free it up before making a new one for the curent buffer
	if(lpPropertySet != NULL)
	{
		lpPropertySet->Release();
		lpPropertySet = NULL;
	}

	// QI for the property set interface
	if( FAILED( hr = lpSecondaryBuffer->QueryInterface( IID_IKsPropertySet,
						(void**)&lpPropertySet)))
	{
		MessageBox( hWndDemo, "Couldn't QI for Property set interface",
				"QI PropertySet failed", MB_OK | MB_ICONSTOP );
	}
	else
	{	
		// check if our listener related property set is avalible and supports everything.
		if( FAILED( hr = lpPropertySet->QuerySupport(DSPROPSETID_EAX_ReverbProperties,
			DSPROPERTY_EAX_ALL, &ulSupport)))
		{
			MessageBox( hWndDemo, "Query support call failure",
					"Query support Error", MB_OK | MB_ICONSTOP );
		}
		else
		{
			// check if we can get and set them.
			if ((ulSupport &(KSPROPERTY_SUPPORT_GET|KSPROPERTY_SUPPORT_SET)) != (KSPROPERTY_SUPPORT_GET|KSPROPERTY_SUPPORT_SET))
			{
				MessageBox( hWndDemo, "can't get and set all properties in Property set interface",
						"No EAX Support", MB_OK | MB_ICONSTOP );
			}
			else
			{
				// check if our Buffer related property set is avalible and supports everything.
				if( FAILED( hr = lpPropertySet->QuerySupport(DSPROPSETID_EAXBUFFER_ReverbProperties,
					DSPROPERTY_EAXBUFFER_ALL, &ulSupport)))
				{
					MessageBox( hWndDemo, "Query support call failure",
							"Query support Error", MB_OK | MB_ICONSTOP );
				}
				else
				{
					// check if we can get and set them.
					if ((ulSupport &(KSPROPERTY_SUPPORT_GET|KSPROPERTY_SUPPORT_SET)) != (KSPROPERTY_SUPPORT_GET|KSPROPERTY_SUPPORT_SET))
					{
						MessageBox( hWndDemo, "can't get and set all properties in Property set interface",
								"No EAX Support", MB_OK | MB_ICONSTOP );
					}
					else
					{
						// everything is avalible.
						bEaxOK = TRUE;
					}
				}
			}
		}
	}
	return(bEaxOK);
}


/*
 * Just get the file name for a .wav from the user.
 */
BOOL FetchUserWaveFile(HWND hWnd)
{
	char szWAVExt[] = "Wave File (*.wav)\0*.wav\00";
	WORD wReturnVal = 0;
	OPENFILENAME ofnWAV;

	ofnWAV.lStructSize		    = sizeof(OPENFILENAME);
	ofnWAV.hwndOwner		    = hWnd;
	ofnWAV.hInstance		    = hInst;
	ofnWAV.lpstrFilter		    = szWAVExt;
	ofnWAV.lpstrCustomFilter	= NULL;
	ofnWAV.nMaxCustFilter	    = 0L;
	ofnWAV.nFilterIndex 	    = 1L;
	ofnWAV.lpstrFile		    = (LPSTR)szUserFileName;
	ofnWAV.nMaxFile		       	= 128;
	ofnWAV.lpstrInitialDir		= NULL;
	ofnWAV.lpstrTitle		    = NULL;
	ofnWAV.lpstrFileTitle	    = NULL;
	ofnWAV.lpstrDefExt		    = NULL;
	ofnWAV.Flags				= 0;

	wReturnVal = GetOpenFileName((LPOPENFILENAME)&ofnWAV);
	if ( wReturnVal > 0 )
		return TRUE;
	else
		return FALSE;
}


/****************************************************************************   
        Function: OpenSecondaryBuffer()                                

        PURPOSE: Read wave data for playback and put it in a secondary buffer
*****************************************************************************/
BOOL OpenSecondaryBuffer(LPSTR lpFilename)
{
    HMMIO			hmmioOut = NULL;    // handle to open output WAVE file
    MMCKINFO		ckOutRIFF;          // chunk info. for output RIFF chunk
    MMCKINFO		ckOut;              // info. for a chunk in output file
	WAVEFORMATEX	pcmOut;				// Format of the wave
	DSBUFFERDESC    dsbdSecondary;		// description for creating buffer
    HRESULT			hr;
	LPVOID			lpvPtr1;			// pointers for locking buffer.
    DWORD			dwBytes1; 
    LPVOID			lpvPtr2; 
    DWORD			dwBytes2; 
	DSBCAPS			DSBufferCaps;


    /////////////////////////////////////////////////
    // Read the wave playback parameters
    /////////////////////////////////////////////////

    // Open the wave file for reading using buffered I/O.
    hmmioOut = mmioOpen(lpFilename, NULL, MMIO_ALLOCBUF | MMIO_READ);
    if (hmmioOut == NULL)
    {
		MessageBox( hWndDemo, "Cannot read file!","WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }
    
    // Descend the file into the 'RIFF' chunk.
    if (mmioDescend(hmmioOut, &ckOutRIFF, NULL, 0) != 0)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Cannot read file!","WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }

    // Make sure the file is a WAVE file.
    if ((ckOutRIFF.ckid != FOURCC_RIFF) ||
        (ckOutRIFF.fccType != mmioFOURCC('W', 'A', 'V', 'E')))
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Bad Wave Format!","WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }

    // Search the file for for the 'fmt ' chunk.
    ckOut.ckid = mmioFOURCC('f', 'm', 't', ' ');
    if (mmioDescend(hmmioOut, &ckOut, &ckOutRIFF, MMIO_FINDCHUNK) != 0)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Bad Riff Format!","WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }

    // Load the format from the file
    if (mmioRead(hmmioOut, (HPSTR) &pcmOut,
                (long) sizeof(pcmOut)) != (long) sizeof(pcmOut))
    {
        mmioClose(hmmioOut, 0);		
		MessageBox( hWndDemo, "Bad Riff Format!","WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }   
    // Check it is PCM not some compressed format which we do not cope with..
        if(pcmOut.wFormatTag!=WAVE_FORMAT_PCM)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "UnSupported Wave Format!", "WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }

    // Ascend the file out of the 'fmt ' chunk.
    if (mmioAscend(hmmioOut, &ckOut, 0) != 0)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Cannot read file!","WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }

    // Search the file for the 'data' chunk.
    ckOut.ckid = mmioFOURCC('d', 'a', 't', 'a');
    if (mmioDescend(hmmioOut, &ckOut, &ckOutRIFF, MMIO_FINDCHUNK) != 0)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Bad Format in Wave file!", "WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }
    
    // check if there is any data
    if(ckOut.cksize==0)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "No wave data!", "WaveLoad", MB_OK | MB_ICONSTOP );
        return FALSE;
    }

	// make a secondary buffer to load the file to. 
	ZeroMemory( &dsbdSecondary, sizeof(DSBUFFERDESC));
	dsbdSecondary.dwSize = sizeof(DSBUFFERDESC);
	dsbdSecondary.dwBufferBytes = ckOut.cksize;
    dsbdSecondary.dwFlags = DSBCAPS_CTRL3D | DSBCAPS_STATIC;// |DSBCAPS_CTRLPOSITIONNOTIFY;
	dsbdSecondary.lpwfxFormat = &pcmOut;

	if( FAILED( hr = lpDirectSound->CreateSoundBuffer( &dsbdSecondary, &lpSecondaryBuffer, NULL )))
	{
		MessageBox( hWndDemo, "Couldn't create Secondary buffer",
				    "Create Secondary failure", MB_OK | MB_ICONSTOP );
        mmioClose(hmmioOut, 0);
        return FALSE;
	}

	// check if software and report if it is.
	ZeroMemory( &DSBufferCaps, sizeof(DSBCAPS));
	DSBufferCaps.dwSize = sizeof(DSBCAPS);

	if( FAILED( hr = lpSecondaryBuffer->GetCaps( &DSBufferCaps)))
	{
		MessageBox( hWndDemo, "Couldn't get caps on Secondary buffer",
				    "GetCaps failure", MB_OK | MB_ICONSTOP );
	}

	if ((DSBufferCaps.dwFlags & DSBCAPS_LOCSOFTWARE)) 
	{
		//In software for some reason.
		MessageBox( hWndDemo, "The buffer has ended up in software.",
				    "Software Buffer", MB_OK | MB_ICONSTOP );
	}

	hr = lpSecondaryBuffer->Lock(0, ckOut.cksize, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, 0); 
    // If DSERR_BUFFERLOST is returned, restore and retry lock. 
	// this should never realy happen here.
    if(hr == DSERR_BUFFERLOST ) 
	{ 
        lpSecondaryBuffer->Restore(); 
        hr = lpSecondaryBuffer->Lock(0, ckOut.cksize, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, DSBLOCK_ENTIREBUFFER); 
    }

	if (FAILED(hr))
	{
		MessageBox( hWndDemo, "Could not Lock() secondary buffer", "Lock Failed", MB_OK | MB_ICONSTOP );
		mmioClose(hmmioOut, 0);
		lpSecondaryBuffer->Release();
		lpSecondaryBuffer = NULL;
        return FALSE;
	}

	// read in the first section of data to the pointer returned by the lock
    if(mmioRead(hmmioOut, (LPSTR)lpvPtr1, dwBytes1) != (LONG) dwBytes1)
    {
        mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Failed to read data chunk1", "WaveLoad", MB_OK | MB_ICONSTOP );
		lpSecondaryBuffer->Release();
		lpSecondaryBuffer = NULL;
        return(FALSE);
    }

	if(NULL != lpvPtr2) 
	{ 
		// If we have some more to read then read it.
		// don't expect to get here realy as locking from the start should give us only one block
		if(mmioRead(hmmioOut, (LPSTR)lpvPtr2, dwBytes2) 
					!= (LONG) dwBytes2)
		{
			mmioClose(hmmioOut, 0);
		MessageBox( hWndDemo, "Failed to read data chunk2", "WaveLoad", MB_OK | MB_ICONSTOP );
			lpSecondaryBuffer->Release();
			lpSecondaryBuffer = NULL;
			return(FALSE);
		}
	}
		
    // Release the data back to DirectSound. 	
	if (FAILED(hr = lpSecondaryBuffer->Unlock( lpvPtr1, dwBytes1, lpvPtr2, dwBytes2)))
	{
		MessageBox( hWndDemo, "Could not UnLock() secondary buffer",
			"UnLock Failed",MB_OK | MB_ICONSTOP );
		mmioClose(hmmioOut, 0);
		lpSecondaryBuffer->Release();
		lpSecondaryBuffer = NULL;
        return FALSE;
	}
	// we are done with the source file so close it.
    mmioClose(hmmioOut, 0);  

	// get the 3d interface for the buffer
	if (FAILED(hr = lpSecondaryBuffer->QueryInterface(IID_IDirectSound3DBuffer, 
												(void**)&lpDs3dBuffer))) 
	{ 
		MessageBox( hWndDemo, "Could not QI() secondary buffer for DS3D",
			"QI DS3D Failed",MB_OK | MB_ICONSTOP );
	// if 3d fails we will still play but nothing will move...
	}
	else
	{
		// set the 3d parameters.  These are default values apart from the position
		// which is updated in WM_MOUSEMOVE
		if (FAILED(hr = lpDs3dBuffer->SetAllParameters(&Buffer3dProps, DS3D_IMMEDIATE)))
		{
			MessageBox( hWndDemo, "Could not SetAllParams() for DS3D",
				"SetAllParams failed", MB_OK | MB_ICONSTOP );
		}
	}
    return(TRUE);
}

/* Bliter  hBitMap on the hdc device context at the passed coords. performing the */
/* Raster OPeration given with dwRop */
void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart, DWORD dwRop)
	{
	BITMAP	bm;
	HDC			hdcMem;
	POINT		ptSize, ptOrg;

	hdcMem = CreateCompatibleDC(hdc);
	SelectObject(hdcMem, hBitmap);
	SetMapMode(hdcMem, GetMapMode(hdc));

	GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
	ptSize.x = bm.bmWidth;
	ptSize.y = bm.bmHeight;
	DPtoLP(hdcMem, &ptSize,1);

	ptOrg.x = 0;
	ptOrg.y = 0;
	DPtoLP(hdcMem, &ptOrg, 1);

	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y,
				 hdcMem, ptOrg.x, ptOrg.y, dwRop);

	DeleteDC(hdcMem);
	}


