// MixClass.cpp : Defines the entry point for the console application.
//
/********************************************************************/
/*	MixClass.cpp													*/
/*	Peter Harrison, Creative Labs UK, 23/4/99						*/
/********************************************************************/
// 
// Demo code for mixer class
// Try running this with a windows UI mixer in the background
// You should see the sliders move as the volumes are altered
// and as the mixers classes are destroyed (if RememberVolumes 
// is set on!)
// ph 23/4/99


#include "stdafx.h"
#include "pomixer.h"

#define RESTORE_VOLUMES true

int main(int argc, char* argv[])
{
	// Variables
	// Mixer object
	POMixer *pMyMixer;
	// Number of CD sources
	int iNumCDs;
	// Counter
	int iCount, iMixer;
	// Number of mixers available
	int iNumberMixers;
	// Mixer we are working on
	int iMixerNumber;

	// Number of mixers available
	iNumberMixers = GetNumMixDevs();
	for (iMixer = 0; iMixer < iNumberMixers; iMixer++)
	{
		// Create a mixer
		pMyMixer = new POMixer(iMixer, RESTORE_VOLUMES);
		// Setup the mixer to be id MixerNumber, with resetting original volumes ON 
		if (pMyMixer->IsValid())
		{
			// Set up OK - display some caps
			printf("Setup mixer #%d of %d\n", iMixer+1, iNumberMixers);
			printf("%s ver %u.%u supporting %u destination lines\n", 
					pMyMixer->ProductName(), 
					pMyMixer->VersionMajor(),
					pMyMixer->VersionMinor(),
					pMyMixer->GetDestinations());
			printf("%d CD sources -> speakers\n", pMyMixer->GetNumberOfCDSources());
		}
		else
		{
			// Error in setup - bail out
			// To display error messages in classes, set DisplayError to true in the 
			// class' header file
			printf("Unable to setup mixer #%d\n", iMixer);
			return -1;
		}
		// We continue...
		printf("\nPress <enter> to delete the mixer object\n");
		char ch = getchar();

		delete pMyMixer;
	}

	if (iNumberMixers < 1)
	{
		// No audio cards !
		printf("No audio mixers found\n");
		return -1;
	}
	if (iNumberMixers == 1)
		// Only one card - no choice to make
		iMixerNumber = 0;
	else
	{
		// Let the use select which mixer to adjust
		int iInput = 0;
		printf("Please enter mixer number (1 to %d) : ", iNumberMixers);
		scanf("%d", &iInput);
		char ch = getchar();
		if (iInput < 1)
			iInput = 1;
		if (iInput > iNumberMixers)
			iInput = iNumberMixers;
		iMixerNumber = iInput-1;
	}
	// Construct the mixer
	pMyMixer = new POMixer(iMixerNumber, RESTORE_VOLUMES);
	if (!pMyMixer->IsValid())
	{
		printf("Unable to setup mixer #%d\n", iMixerNumber);
		return -1;
	}
	// Display mixer device name
	printf("Setting levels on %s\n", pMyMixer->ProductName());

	// How many possible CD sources are there ? (digital AND analog connected ?)
	iNumCDs = pMyMixer->GetNumberOfCDSources(); 

	// Display the current volume settings (expressed as a percentage)
	for (iCount = 0; iCount < iNumCDs; iCount ++)
		// Each CD source
		printf("CD #%d Vol : %d% \n", iCount, pMyMixer->GetCDVolumePercent(iCount));
	printf("MIDI Vol : %d% \n", pMyMixer->GetMIDIVolumePercent());
	printf("Wave Vol : %d% \n", pMyMixer->GetWaveVolumePercent());
	printf("Main Vol : %d% \n", pMyMixer->GetMainVoumePercent());

	// Set main volume

	if (pMyMixer->SetMainVolumePercent(90))
	{
		printf("Main set volume to 90%\n");
	}
	else
	{
		printf("Failed to set Main volume\n");
	}

	// Set MIDI volume
	if (pMyMixer->SetMIDIVolumePercent(70))
	{
		printf("MIDI set volume to 70%\n");
	}
	else
	{
		printf("Failed to set MIDI volume\n");
	}

	// Set WAVE volume
	if (pMyMixer->SetWaveVolumePercent(50))
	{
		printf("WAVE set volume to 50%\n");
	}
	else
	{
		printf("Failed to set WAVE volume\n");
	}

	// Set CD volume
	for (iCount = 0; iCount < iNumCDs; iCount ++)
		if (pMyMixer->SetCDVolumePercent(iCount, (10 * iCount) + 50))
		{
			printf("CD set volume to %d%\n", (10*iCount)+50);
		}
		else
		{
			printf("Failed to set CD volume\n");
		}

	// Request and display the new settings to prove they have been set
	for (iCount = 0; iCount < iNumCDs; iCount ++)
		printf("CD #%d Vol : %d% \n", iCount, pMyMixer->GetCDVolumePercent(iCount));
	printf("MIDI Vol : %d% \n", pMyMixer->GetMIDIVolumePercent());
	printf("Wave Vol : %d% \n", pMyMixer->GetWaveVolumePercent());
	printf("Main Vol : %d% \n", pMyMixer->GetMainVoumePercent());

	// Wait for <enter>
	printf("\nPress <enter> to delete the mixer object\n");
	char ch = getchar();

	// Delete the mixer
	// If RESTORE_VOLUMES is true, the old settings will be restored here
	// bye bye...
	delete pMyMixer;
	return 0;
}
