// POMixer.cpp: implementation of the POMixer class.
//
//////////////////////////////////////////////////////////////////////

/********************************************************************/
/*	MixClass - POMixer.cpp											*/
/*	Peter Harrison, Creative Labs UK, 23/4/99						*/
/********************************************************************/

#include "stdafx.h"
#include "pomixer.h"

// exported global variable
int g_iNumMixDevs;

int GetNumMixDevs()
// exported mixer number function
{
	return mixerGetNumDevs();
}
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

POMixer::POMixer(UINT uMixerID, BOOL bRememberVolumes)
{
	// Contruct a new POMixer
	int iDCount, iSCount;
	g_iNumMixDevs = mixerGetNumDevs();
	m_uMixerID = -1;
	m_hMixer = NULL;
	m_iCDSources = 0;
	m_iDestinations = 0;
	for (iDCount = 0; iDCount < MAX_DESTINATIONS; iDCount++)
	{
		// Set all MixerLine pointers to null
		m_aDestMixerLines[iDCount] = NULL;
		for (iSCount = 0; iSCount < MAX_SOURCES; iSCount++)
			m_aSourceMixerLines[iDCount][iSCount] = NULL;
	}
	Initialise(uMixerID, bRememberVolumes);
}

POMixer::~POMixer()
{
	int iDCount, iSCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount++)
	// Loop through destinations
	{
		POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
		if (TempDestLine != NULL)
		{
			for (iSCount = 0; iSCount < TempDestLine->GetConnections(); iSCount++)
			// Loop through sources
			{
				// delete source line
				if (m_aSourceMixerLines[iDCount][iSCount] != NULL)
					delete (m_aSourceMixerLines[iDCount][iSCount]);
				m_aSourceMixerLines[iDCount][iSCount] = NULL;
			}
			delete m_aDestMixerLines[iDCount];
			m_aDestMixerLines[iDCount] = NULL;
		}
	}
}

char POMixer::VersionMajor()
{
	// Version number major
	if (m_uMixerID == -1)
		return 0;
	return HIBYTE(m_Caps.vDriverVersion);
}

char POMixer::VersionMinor()
{
	// Version number minor
	if (m_uMixerID == -1)
		return 0;
	return LOBYTE(m_Caps.vDriverVersion);
}

char *POMixer::ProductName()
{
	// Product name string
	if (m_uMixerID == -1)
		return NULL;
	char *szReturns;
	szReturns = (char *)malloc(MAXPNAMELEN);
	strcpy(szReturns, m_Caps.szPname);
	return szReturns;

}

DWORD POMixer::GetDestinations()
{
	// Number of destinations provided
	if (m_uMixerID == -1)
		return -1;
	return (m_iDestinations);
}

BOOL POMixer::Initialise(UINT uMixerID, BOOL bRememberVolumes)
{
	// Variables
	
	MMRESULT			mmResult;
	int					iDCount, iSCount;

	if ((int)m_uMixerID != -1)
	{
		Error("SetupMixer()", "Mixer already initialised");
		return false;
	}

	m_bRememberVolumes = bRememberVolumes;
	// First, check the device represented by uMixerID exists
	if ((int)uMixerID > (g_iNumMixDevs + 1))
	{
		// uMixerID greater than Number of devices
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Mixer id %u does not exist", uMixerID);
		Error("SetupMixer()", ErrorMessage);
		return false;
	}
	m_uMixerID = uMixerID;

	// Next, get CAPS
	if ((mmResult = mixerGetDevCaps(m_uMixerID, &m_Caps, sizeof(MIXERCAPS))) != 0)
	{
		// Could not get caps
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get caps for mixer id %u ", uMixerID);
		Error("SetupMixer()", ErrorMessage);
		return false;
	}

	if ((mmResult = mixerOpen(&m_hMixer, uMixerID, 0L, 0L, 
							 MIXER_OBJECTF_MIXER)) != 0)
	{
		// Could not get caps - process error
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not open mixer id %u ", uMixerID);
		Error("SetupMixer()", ErrorMessage);
		return false;
	}

	m_iDestinations = (int)m_Caps.cDestinations;

	// Loop through destination lines
	for (iDCount = 0; iDCount < m_iDestinations; iDCount ++)
	{
		// Variables

		int					iSources;
		POMixerLine			*tempDestLine;

		// Get the destination line

		tempDestLine = new POMixerLine(iDCount, (HMIXEROBJ)m_hMixer, uMixerID, m_bRememberVolumes);

		// Number of sources for this destination
		
		iSources = tempDestLine->GetConnections();
		
		// Save the info structure
		
		m_aDestMixerLines[iDCount] = tempDestLine;
		if (DisplayInfo) printf("Line #%d has %d source(s)\n", iDCount, iSources);

		// Loop through source lines
		for (iSCount = 0; iSCount < (int)iSources; iSCount++)
		{
  			// Variables
			POMixerLine				*tempSourceLine;


			tempSourceLine = new POMixerLine(iDCount, iSCount, 
						(HMIXEROBJ)m_hMixer, uMixerID, tempDestLine->IsMainOut(), m_bRememberVolumes);
			m_aSourceMixerLines[iDCount][iSCount] = tempSourceLine;
			if (tempSourceLine->CouldBeCD() && tempSourceLine->IsMainOut())
			{
				// This source could represent CD audio
				m_iCDSources ++;
			}
		}
	}
	return true;
}

int	POMixer::GetNumberOfCDSources()
{
	return m_iCDSources;
}

BOOL POMixer::SetMIDIVolumePercent(int iPercent)
{
	// Sets the volume of the FIRST MIDI source -> SPK destination encountered
	int iDCount, iSCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			for (iSCount = 0; iSCount < (int)TempDestLine->GetConnections(); iSCount ++)
			{
				if (m_aSourceMixerLines[iDCount][iSCount]->isMIDISource())
				{
					// Found MIDI -> Speakers
					POMixerLine *TempSourceLine = 
						m_aSourceMixerLines[iDCount][iSCount];
					return TempSourceLine->SetVolumePercent(iPercent);
				}
			}
		}
	}
	return false;
}

BOOL POMixer::SetWaveVolumePercent(int iPercent)
{
	// Sets the volume of the FIRST WAVE source -> SPK destination encountered
	int iDCount, iSCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			for (iSCount = 0; iSCount < (int)TempDestLine->GetConnections(); iSCount ++)
			{
				if (m_aSourceMixerLines[iDCount][iSCount]->IsWaveSource())
				{
					// Found WAVE -> Speakers
					POMixerLine *TempSourceLine = 
						m_aSourceMixerLines[iDCount][iSCount];
					return TempSourceLine->SetVolumePercent(iPercent);
				}
			}
		}
	}
	return false;
}

BOOL POMixer::SetCDVolumePercent(int iCDNumber, int iPercent)
{
	// Sets the volume of the iCDNumber-th CD source -> SPK destination encountered
	int iDCount, iSCount, iCDEncountered = -1;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			for (iSCount = 0; iSCount < (int)TempDestLine->GetConnections(); iSCount ++)
			{
				if (m_aSourceMixerLines[iDCount][iSCount]->CouldBeCD())
				{
					// Found CD -> Speakers
					POMixerLine *TempSourceLine = 
						m_aSourceMixerLines[iDCount][iSCount];
					iCDEncountered++;
					if (iCDEncountered == iCDNumber)
						return TempSourceLine->SetVolumePercent(iPercent);
				}
			}
		}
	}
	return false;
}

BOOL POMixer::SetMainVolumePercent(int iPercent)
{
	// Sets the volume of the FIRST SPK destination encountered
	int iDCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			return (TempDestLine->SetVolumePercent(iPercent));
		}
	}
	return false;
}

int POMixer::GetMIDIVolumePercent()
{
	// Gets the volume of the FIRST MIDI source -> SPK destination encountered
	int iDCount, iSCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			for (iSCount = 0; iSCount < (int)TempDestLine->GetConnections(); iSCount ++)
			{
				if (m_aSourceMixerLines[iDCount][iSCount]->isMIDISource())
				{
					// Found MIDI -> Speakers
					POMixerLine *TempSourceLine = 
						m_aSourceMixerLines[iDCount][iSCount];
					return TempSourceLine->GetVolumePercent();
				}
			}
		}
	}
	return -1;
}

int POMixer::GetWaveVolumePercent()
{
	// Gets the volume of the FIRST WAVE source -> SPK destination encountered
	int iDCount, iSCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			for (iSCount = 0; iSCount < (int)TempDestLine->GetConnections(); iSCount ++)
			{
				if (m_aSourceMixerLines[iDCount][iSCount]->IsWaveSource())
				{
					// Found WAVE -> Speakers
					POMixerLine *TempSourceLine = 
						m_aSourceMixerLines[iDCount][iSCount];
					return TempSourceLine->GetVolumePercent();
				}
			}
		}
	}
	return -1;
}

int POMixer::GetCDVolumePercent(int iCDNumber)
{
	// Sets the volume of the iCDNumber-th CD source -> SPK destination encountered
	int iDCount, iSCount, iCDEncountered = -1;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			for (iSCount = 0; iSCount < (int)TempDestLine->GetConnections(); iSCount ++)
			{
				if (m_aSourceMixerLines[iDCount][iSCount]->CouldBeCD())
				{
					// Found CD -> Speakers
					POMixerLine *TempSourceLine = 
						m_aSourceMixerLines[iDCount][iSCount];
					iCDEncountered++;
					if (iCDEncountered == iCDNumber)
						return TempSourceLine->GetVolumePercent();
				}
			}
		}
	}
	return -1;
}

int POMixer::GetMainVoumePercent()
{
	// Sets the volume of the FIRST SPK destination encountered
	int iDCount;
	for (iDCount = 0; iDCount < (int)GetDestinations(); iDCount ++)
	{
		// Loop through destinations
		if (m_aDestMixerLines[iDCount]->IsMainOut())
		{
			// Found speaker destination
			POMixerLine *TempDestLine = m_aDestMixerLines[iDCount];
			return (TempDestLine->GetVolumePercent());
		}
	}
	return -1;
}

void POMixer::Error(char *Function, char *Message)
{
	// Private error display function
	if (!DisplayErrors)
		return;
	printf("ERROR ");
	printf(Function);
	printf(" : ");
	printf(Message);
	printf("\n");
}
