// POMixer.h: interface for the POMixer class.
//
//////////////////////////////////////////////////////////////////////

/********************************************************************/
/*	MixClass - POMixer.h											*/
/*	Peter Harrison, Creative Labs UK, 23/4/99						*/
/********************************************************************/

//#if !defined(AFX_POMIXER_H__6F767826_E784_11D2_B576_00AA00AA3C58__INCLUDED_)
//#define AFX_POMIXER_H__6F767826_E784_11D2_B576_00AA00AA3C58__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#if !defined pomixer
#define pomixer

// Maximum destination lines the class can support in one mixer
#define MAX_DESTINATIONS 20
// Maximum source lines per destination the class can support in one mixer
#define MAX_SOURCES 30

// Uses POMixerLine class
#include "POMixerLine.h"

// Display errors to standard console
#define DisplayErrors false
// Display info to standard console
#define DisplayInfo false

// Number of mixers in system - only set when a mixer has been declared
extern int GetNumMixDevs();

class POMixer  
{
public:
	// Methods
	// Constructor
	POMixer(UINT uMixerID, BOOL bRememberVolumes);
	// Destructor
	~POMixer();
	// Is this mixer set up and valid ?
	BOOL IsValid() {return (m_uMixerID != -1);}
	// Major version number
	char VersionMajor();
	// Minor version number
	char VersionMinor();
	// Name of audio device this mixer controls
	char *ProductName();
	// Number of destinations controlled
	DWORD GetDestinations();
	// Number of CD Sources on main (speaker) destination
	int	GetNumberOfCDSources();
	// Set volume of first source MIDI -> SPKR
	BOOL SetMIDIVolumePercent(int iPercent);
	// Set volume of first source WAVE -> SPKR
	BOOL SetWaveVolumePercent(int iPercent);
	// Set volume of source CD -> SPKR
	// index iCDNumber is range 0 .. GetNumberOfCDSources-1
	BOOL SetCDVolumePercent(int iCDNumber, int iPercent);
	// Set volume of first SPKR destination
	BOOL SetMainVolumePercent(int iPercent);

	// Returns volume of first source MIDI -> SPKR
	int GetMIDIVolumePercent();
	// Returns volume of first source WAVE -> SPKR
	int GetWaveVolumePercent();
	// Returns volume of source CD -> SPKR,
	// index iCDNumber is range 0 .. GetNumberOfCDSources-1
	int GetCDVolumePercent(int iCDNumber);
	// Returns volume of first SPKR destination
	int GetMainVoumePercent();
	// Parameters
private:
	// Methods

	// Display error to standard
	void Error(char *Function, char *Message);
	// Initialise a mixer
	// uMixerID is the ID of the mixer to be initialised
	// if bRememberVolumes is set, the volume levels will be restored when the 
	// mixer is destroyed
	BOOL Initialise(UINT uMixerID, BOOL bRememberVolumes);

	// Parameters

	// Number of destinations on this mixer
	int					m_iDestinations;
	// Number of possible CD sources on this mixer
	int					m_iCDSources;
	// Has setup been run yet ?
	BOOL				m_bSetup;
	// Index of mixer
	UINT				m_uMixerID;
	// Mixer's caps structure
	MIXERCAPS			m_Caps;
	// Mixer's handle
	HMIXER				m_hMixer;
	// Whether original volumes are being stored / restored
	BOOL				m_bRememberVolumes;
	// Array of destination line objects
	POMixerLine			*m_aDestMixerLines[MAX_DESTINATIONS];
	// Array of source line objects
	POMixerLine			*m_aSourceMixerLines[MAX_DESTINATIONS][MAX_SOURCES];
};

#endif //pomixer

//#endif // !defined(AFX_POMIXER_H__6F767826_E784_11D2_B576_00AA00AA3C58__INCLUDED_)
