// POMixerLine.cpp: implementation of the POMixerLine class.
//
//////////////////////////////////////////////////////////////////////

/********************************************************************/
/*	MixClass - POMixerLine.cpp										*/
/*	Peter Harrison, Creative Labs UK, 23/4/99						*/
/********************************************************************/

#include "stdafx.h"
#include "POMixerLine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

POMixerLine::POMixerLine(HMIXEROBJ hmxobj, UINT uMixerID, BOOL bRememberVolume)
// Generic constructor
{
	m_MixerLine.cbStruct = sizeof(MIXERLINE);
	m_uMixerID = uMixerID;
	m_hmxobj = hmxobj;
	m_bRememberVolume = bRememberVolume;
	m_bIsMIDI = false;
	m_bIsWave = false;
	m_bCouldBeCD = false;
	m_bHasVolume = false;
	m_bIsMainOut = false;
}

POMixerLine::POMixerLine(DWORD dwD, HMIXEROBJ hmxobj, UINT uMixerID, 
						 BOOL bRememberVolume)
// Constructor for destination line
{
	m_bIsMIDI = false;
	m_bIsWave = false;
	m_bCouldBeCD = false;
	m_bHasVolume = false;
	m_bIsMainOut = false;
	m_MixerLine.cbStruct = sizeof(MIXERLINE);
	m_MixerLine.dwDestination = dwD;
	m_LineType = Destination;
	m_hmxobj = hmxobj;
	m_uMixerID = uMixerID;
	RequestMixerLine();
	m_bHasVolume = RequestVolumeControl();
	m_bIsMainOut = (m_MixerLine.dwComponentType == MIXERLINE_COMPONENTTYPE_DST_SPEAKERS);
	if (DISPLAY_INFO) printf(" Vol : %d% ", GetVolumePercent());
	if ((m_bRememberVolume = bRememberVolume) == TRUE)
		m_dwVolumeAtStart = GetVolume();
	if (m_bIsMainOut)
	{
		if (DISPLAY_INFO) printf(" <SPK> ");
	}
	if (DISPLAY_INFO) printf("\n");
}

POMixerLine::POMixerLine(DWORD dwD, DWORD dwS, HMIXEROBJ hmxobj, UINT uMixerID, 
						 BOOL bIsMainOut, BOOL bRememberVolume)
// Constructor for source line
{
	m_bIsMIDI = false;
	m_bIsWave = false;
	m_bCouldBeCD = false;
	m_bHasVolume = false;
	m_bIsMainOut = false;
	m_MixerLine.cbStruct = sizeof(MIXERLINE);
	m_MixerLine.dwDestination = dwD;
	m_MixerLine.dwSource = dwS;
	m_LineType = Source;
	m_hmxobj = hmxobj;
	m_uMixerID = uMixerID;
	RequestMixerLine();
	m_bHasVolume = RequestVolumeControl();
	m_bIsMainOut = bIsMainOut;
	if (DISPLAY_INFO) printf(" Vol : %d% ", GetVolumePercent());
	if ((m_bRememberVolume = bRememberVolume) == TRUE)
		m_dwVolumeAtStart = GetVolume();
	if (m_bIsMainOut)
	{
		if (DISPLAY_INFO) printf(" <SPK> ");
	}
	if (DISPLAY_INFO) printf("\n");
}

POMixerLine::~POMixerLine()
// Deconstructor
{
	if (m_bRememberVolume)
		SetVolume(m_dwVolumeAtStart);
}

BOOL POMixerLine::RequestMixerLine()
{
	// Return value
	MMRESULT mmResult;
	// Flags for mixerGetLintInfo
	DWORD fdwInfo;

	// Set the flags
	switch (m_LineType)
	{
	case Source:
		{
			fdwInfo = MIXER_GETLINEINFOF_SOURCE;
			break;
		}
	case Destination:
		{
			fdwInfo = MIXER_GETLINEINFOF_DESTINATION;
			break;
		}
	}

	// Request the structure
	if ((mmResult = mixerGetLineInfo(m_hmxobj, &m_MixerLine, fdwInfo)) != 0)
	{
		// Could not get line info - process error
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get source line %d to destination %d on mixer id %u ", 
				m_MixerLine.dwDestination, 
				m_MixerLine.dwSource, m_uMixerID);
		Error("RequestMixerLine()", ErrorMessage);
		return false;
	}

	// OK, we got the info
	if (m_LineType == Destination)
	{
		if (DISPLAY_INFO) printf("Dest #%d Controls %d: %s", 
			m_MixerLine.dwDestination, 
			m_MixerLine.cControls,
			GetLongName());
	}
	else
	{
		if (DISPLAY_INFO) printf("\tDest #%d Source #%d Controls %d: %s", 
			m_MixerLine.dwDestination, 
			m_MixerLine.dwSource, 
			m_MixerLine.cControls,
			GetLongName()); 
	}
	m_dwLineID = m_MixerLine.dwLineID;
	// Set type properties -
	// Only worth setting if we have a volume control otherwise couldn't
	// set level anyway !
	m_bCouldBeCD = 
		(
		// Criteria for CD
		m_MixerLine.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC
		||(strstr(GetLongName(), "CD") != NULL)
		||(strstr(GetShortName(), "CD") != NULL)
		);
	if (m_bCouldBeCD)
	{
		if (DISPLAY_INFO) printf(" <CD> ");
		return true;
	}
	m_bIsMIDI = 
	(
		// Criteria for MIDI
		m_MixerLine.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER
	);
	if (m_bIsMIDI)
	{
		if (DISPLAY_INFO) printf(" <MIDI> ");
		return true;
	}
	m_bIsWave = 
	(
		// Criteria for Wave out
		m_MixerLine.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT
	);
	if (m_bIsWave)
	{
		if (DISPLAY_INFO) printf(" <Wave> ");
		return true;
	}
	return true;
}

BOOL POMixerLine::RequestVolumeControl()
{
	// Return value
	MMRESULT mmResult;
	// Flags for mixerGetLineControls
	DWORD fdwControls;
	// Temporary controller data

	// Set flags
	fdwControls = MIXER_GETLINECONTROLSF_ONEBYTYPE;

	// Set up structure
	// Size
	m_MixerLineControls.cbStruct = sizeof(m_MixerLineControls);
	// ID was determined earlier at construction
	m_MixerLineControls.dwLineID = m_dwLineID;
	// Looking for a volume control
	m_MixerLineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
	// Sizes of sub structures
	m_MixerLineControls.cbmxctrl = sizeof(m_MixerVolumeControl);
	m_MixerLineControls.pamxctrl = &m_MixerVolumeControl;
	m_MixerVolumeControl.cbStruct = sizeof(m_MixerVolumeControl);
	// Request the structure
	if ((mmResult = mixerGetLineControls(m_hmxobj, 
										 &m_MixerLineControls, 
										 fdwControls)) != 0)
	{
		// Could not get line info - process error
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get mixer line volume");
		Error("RequestMixerControls()", ErrorMessage);
		return false; 
	}
	// OK, we got the control
	m_dwVolumeControlID = m_MixerVolumeControl.dwControlID;
	return true;
}

DWORD POMixerLine::GetVolume()
{
	// Return value
	MMRESULT mmResult;
	// Flags for mixerGetLineControls
	DWORD fdwDetails;
	// Temporary control details structure
	MIXERCONTROLDETAILS ControlDetails;
	MIXERCONTROLDETAILS_UNSIGNED ControlDetailsVolume;

	// Check for volume controller
	if (!m_bHasVolume)
	{
		// Cannot control volume on this line
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get Volume Control for source line %d to destination %d on mixer id %u ", 
				m_MixerLine.dwDestination, 
				m_MixerLine.dwSource, m_uMixerID);
		Error("GetVolume()", ErrorMessage);
		return 0;
	}

	
	// Set up flag
	fdwDetails = MIXER_GETCONTROLDETAILSF_VALUE;
	// Set up structures
	// Size
	ControlDetails.cbStruct = sizeof(ControlDetails);
	// Got ID earlier during RequestVolumeControl
	ControlDetails.dwControlID = m_dwVolumeControlID;
	// Uniform volume control please
	ControlDetails.cChannels = 1;
	ControlDetails.cMultipleItems = 0;
	// Sub structure
	ControlDetails.cbDetails = sizeof(ControlDetailsVolume);
	ControlDetails.paDetails = (LPVOID)&ControlDetailsVolume;

	// Request the structure
	if ((mmResult = mixerGetControlDetails(m_hmxobj, 
										   &ControlDetails,
										   fdwDetails)) != 0)
	{
		// Could not get line info - process error
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get mixer line volume");
		Error("GetVolume()", ErrorMessage);
		return 0;
	}

	// OK, we got the value
	return (((LPMIXERCONTROLDETAILS_UNSIGNED)ControlDetails.paDetails)->dwValue);
}

BOOL POMixerLine::SetVolume(DWORD dwVolume)
{
	// Return value
	MMRESULT mmResult;
	// Flags for mixerGetLineControls
	DWORD fdwDetails;
	// Temporary control details structure
	MIXERCONTROLDETAILS ControlDetails;
	MIXERCONTROLDETAILS_UNSIGNED ControlDetailsVolume;

	// Check for volume controller
	if (!m_bHasVolume)
	{
		// Cannot control volume on this line
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get Volume Control for source line %d to destination %d on mixer id %u ", 
				m_MixerLine.dwDestination, 
				m_MixerLine.dwSource, m_uMixerID);
		Error("SetVolume()", ErrorMessage);
		return false;
	}

	
	// Set up flag
	fdwDetails = MIXER_GETCONTROLDETAILSF_VALUE;
	// Set up structures
	// Size
	ControlDetails.cbStruct = sizeof(ControlDetails);
	// Got ID earlier during RequestVolumeControl
	ControlDetails.dwControlID = m_dwVolumeControlID;
	// Uniform volume control please
	ControlDetails.cChannels = 1;
	ControlDetails.cMultipleItems = 0;
	// Sub structure
	ControlDetails.cbDetails = sizeof(ControlDetailsVolume);
	ControlDetails.paDetails = (LPVOID)&ControlDetailsVolume;
	ControlDetailsVolume.dwValue = dwVolume;
	// Request the structure
	if ((mmResult = mixerSetControlDetails(m_hmxobj, 
										   &ControlDetails,
										   fdwDetails)) != 0)
	{
		// Could not get line info - process error
		char ErrorMessage[255];
		sprintf(ErrorMessage, "Could not get mixer line volume");
		Error("SetVolume()", ErrorMessage);
		return false;
	}

	// OK, we set the value
	return true;
}


int POMixerLine::GetConnections()
{
	// Return number of connections if this is a dest, otherwise 0
	if (m_LineType == Source)
		return 0;
	return (int)m_MixerLine.cConnections;
}

int POMixerLine::GetControls()
{
	// Return number of controls
	return (int)m_MixerLine.cControls;
}

DWORD POMixerLine::PercentToDWORD(char cPercent)
{
	// Convert a Percentage to an absolute unsigned value according
	// to this line's volume control's bounds
	DWORD dwRange, dwOnePercent;

	// First make sure the Percent is in range
	if (cPercent > 100)
		cPercent = 100;
	if (cPercent < 0)
		cPercent = 0;
	dwRange = (m_MixerVolumeControl.Bounds.dwMaximum - 
			   m_MixerVolumeControl.Bounds.dwMinimum);
	dwOnePercent = (DWORD)((float)dwRange / 100.0);
	return (m_MixerVolumeControl.Bounds.dwMinimum + 
			((DWORD)cPercent * dwOnePercent));
}				

char POMixerLine::DWORDToPercent(DWORD dwValue)
	// Convert an absolute unsigned value to a percentage according
	// to this line's volume control's bounds
{
	DWORD dwRange, dwOnePercent, dwDifference;

	dwRange = (m_MixerVolumeControl.Bounds.dwMaximum - 
			   m_MixerVolumeControl.Bounds.dwMinimum);
	dwOnePercent = (DWORD)((float)dwRange / 100.0);
	dwDifference = (dwValue - m_MixerVolumeControl.Bounds.dwMinimum);
	return ((char)((float)dwDifference / (float)dwOnePercent));
}

char POMixerLine::GetVolumePercent()
{
	// Returns line volume by percent
	return(DWORDToPercent(GetVolume()));
}

BOOL POMixerLine::SetVolumePercent(char cVolume)
{
	// Sets line volume as a percent
	return(SetVolume(PercentToDWORD(cVolume)));
}

void POMixerLine::Error(char *Function, char *Message)
// Private error display
{
	if (!DISPLAY_ERRORS)
		return;
	printf("\nPOMixerLine ERROR ");
	printf(Function);
	printf(" : ");
	printf(Message);
	printf("\n");
}
