// POMixerLine.h: interface for the POMixerLine class.
//
//////////////////////////////////////////////////////////////////////

/********************************************************************/
/*	MixClass - POMixerLine.h										*/
/*	Peter Harrison, Creative Labs UK, 23/4/99						*/
/********************************************************************/

#if !defined(AFX_POMIXERLINE_H__CA3CCAA0_EC31_11D2_B576_00AA00AA3C58__INCLUDED_)
#define AFX_POMIXERLINE_H__CA3CCAA0_EC31_11D2_B576_00AA00AA3C58__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"

#define DISPLAY_ERRORS false
#define DISPLAY_INFO false

enum LineTypes
{
	Destination,
	Source
} typedef LINETYPE;

class POMixerLine  
{
public:
	// Methods

	// Constructors
	// Basic constructor
	POMixerLine(HMIXEROBJ hmxobj, UINT uMixerID, BOOL bRememberVolumes);
	// Contructor for a Destination line
	POMixerLine(DWORD dwD, HMIXEROBJ hmxobj, UINT uMixerID, BOOL bRememberVolumes);
	// Constructor for a Source line
	POMixerLine(DWORD dwD, DWORD dwS, HMIXEROBJ hmxobj, UINT uMixerID, BOOL bIsMainOut, BOOL bRememberVolumes);

	// Destructors
	~POMixerLine();

	// Return information
	// Number of connections if this is a dest, else returns 0 for source
	int GetConnections();
	// Number of controls on this line
	int GetControls();
	// Type of line ie. Source or Destination
	LINETYPE GetLineType()
		{ return m_LineType; };
	// Short name of line
	char *GetShortName()
		{ return m_MixerLine.szShortName ;};
	// Full name of line
	char *GetLongName()
		{ return m_MixerLine.szName ;};
	// ID of line
	DWORD GetLineID()
		{ return m_MixerLine.dwLineID ;};
	// Whether or not Volume can be controlled
	BOOL CanControlVolume()
		{ return m_bHasVolume;};
	// Whether or not line is associated with SPEAKERS destination
	BOOL IsMainOut()
		{ return m_bIsMainOut;};
	// Whether the line could be a MIDI source
	BOOL isMIDISource()
		{ return m_bIsMIDI;};
	// Whether the line could be a Wave source
	BOOL IsWaveSource()
		{ return m_bIsWave;};
	// Whether the line could be a CD source
	BOOL CouldBeCD()
		{ return m_bCouldBeCD;};
	// Volume by percent
	char GetVolumePercent();
	// Set information
	// Volume by percent
	BOOL SetVolumePercent(char cVolume);

	// Paramters


private:
	// Methods

	// Setup - request the line info
	BOOL RequestMixerLine();
	// Setup - request the line's volume control
	BOOL RequestVolumeControl();
	// Convert a percentage to an unsigned value depending on bounds
	DWORD PercentToDWORD(char cPercent);
	// Convert an unsigned value to a percentage depending on bounds
	char DWORDToPercent(DWORD dwValue);
	// Print error text
	void Error(char *Function, char *Message);
	// Set volume by unsigned value
	BOOL SetVolume(DWORD dwVolume);
	// Get unsigned value for volume
	DWORD GetVolume();

	// Parameters

	// The handle of the mixer this line belongs to
	HMIXEROBJ				m_hmxobj;
	// The index of the mixer this line belongs to
	UINT					m_uMixerID;
	// Line type (dest or source) of POMixerLine
	LINETYPE				m_LineType;
	// Mixer line information
	MIXERLINE				m_MixerLine;
	// Mixer line volume control information
	MIXERCONTROL			m_MixerVolumeControl;
	// Mixer line control information
	MIXERLINECONTROLS		m_MixerLineControls;
	// Line ID number
	DWORD					m_dwLineID;
	// Line's volume ID number
	DWORD					m_dwVolumeControlID;
	// Volume when constructed, for reseting 
	DWORD					m_dwVolumeAtStart;
	// Remember to reset volume at the end
	BOOL					m_bRememberVolume;
	// Line could be a CD source
	BOOL					m_bCouldBeCD;
	BOOL					m_bHasVolume;
	BOOL					m_bIsMainOut;
	BOOL					m_bIsWave;
	BOOL					m_bIsMIDI;


};

#endif // !defined(AFX_POMIXERLINE_H__CA3CCAA0_EC31_11D2_B576_00AA00AA3C58__INCLUDED_)
