#define INITGUID     // or link with dxguid.lib

#include <windows.h>
#include <dsound.h>
#include "vmanpset.h"
#include <stdio.h>

int main(void)
{
	LPDIRECTSOUND		lpDS = NULL;				// Direct Sound Object
	LPDIRECTSOUNDBUFFER	lpdsbPrimary = NULL;		// Primary Buffer
	LPDIRECTSOUNDBUFFER lpSecondaryBuffer = NULL;	// Dummy secondary buffer
	LPKSPROPERTYSET		lpPropertySet = NULL;		// Temporary Property Set Interface
	LPDIRECTSOUNDBUFFER Buffers[40] = {0};			// Application's wave files
	LPKSPROPERTYSET		PropSet[40] = {0};			// PS Interfaces for each wavefile
	
	DSBUFFERDESC		dsbdSecondary;				// Secondary buffer description
	DSBUFFERDESC		dsbdPrimary;				// Primary buffer descpription
	DSBCAPS				dsbCaps;					// Buffer's Caps
	DSCAPS				dsCaps;						// Direct Sound Caps
	
	WAVEFORMATEX		wfx;	// WAVE structure
	
	HWND		hWnd;			// Window handle

	HRESULT		hr;
	ULONG		ulSupport = 0;
	
	VmMode		Mode;
	
	int			loop;
	

	if(FAILED(hr = DirectSoundCreate(NULL,&lpDS,NULL)))
	{
		goto ID_Exit;
	}
	
	printf("Created Direct Sound Object\n");

	hWnd = GetDesktopWindow();
	
	if(FAILED(hr = lpDS->SetCooperativeLevel(hWnd, DSSCL_EXCLUSIVE)))
	{
		goto ID_ReleaseDirectSound;
	}
  
	printf("Set Co-operative level\n");

	// Create the primary buffer.
	memset(&dsbdPrimary, 0, sizeof(DSBUFFERDESC));
    dsbdPrimary.dwSize = sizeof(DSBUFFERDESC);
    dsbdPrimary.dwFlags = DSBCAPS_CTRL3D | DSBCAPS_PRIMARYBUFFER;

    if( FAILED( hr = lpDS->CreateSoundBuffer( &dsbdPrimary, &lpdsbPrimary, NULL )))
	{
		goto ID_ReleaseDirectSound;
	}

	printf("Created Primary Buffer\n");

	memset(&dsbCaps, 0, sizeof(DSBCAPS));
	dsbCaps.dwSize = sizeof(DSBCAPS);

	lpdsbPrimary->GetCaps(&dsbCaps);

	if (dsbCaps.dwFlags & DSBCAPS_LOCHARDWARE)
	{
		printf("Sound card supports hardware accelerated 3D audio\n");
	}
	else
	{
		printf("No 3D Audio Hardware acceleration present\n");
		goto ID_ReleasePrimaryBuffer;
	}

	// Set Primary buffer to 16bit 44.1KHz. 
	memset(&wfx, 0,sizeof(WAVEFORMATEX));
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nChannels = 2;
	wfx.nSamplesPerSec = 44100;
	wfx.nAvgBytesPerSec = 176400;
	wfx.nBlockAlign = 4;
	wfx.wBitsPerSample = 16;	
	wfx.cbSize = 0;
	if( FAILED( hr = lpdsbPrimary->SetFormat( &wfx)))
	{
		printf("Failed to set Primary buffer format\n");
		// If we fail with our format we just go with the default rather than bombing out.
	}
	
	// Now create Dummy buffer to be able to create a property set interface

	memset(&dsbdSecondary, 0, sizeof(DSBUFFERDESC));
	memset(&wfx, 0, sizeof(WAVEFORMATEX));

	// Any format should do, e.g 11kHz 16 bit mono
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nChannels = 1;
	wfx.nSamplesPerSec = 11050;
	wfx.nAvgBytesPerSec = 22100;
	wfx.nBlockAlign = 2;
	wfx.wBitsPerSample = 16;
	wfx.cbSize = 0;

	// Set the dwBufferBytes size to 32 (problems can occur if this value is too small)
	dsbdSecondary.dwSize = sizeof(DSBUFFERDESC);
	dsbdSecondary.dwBufferBytes = 32;
	// Don't set DSBCAPS_CTRLPOSITIONNOTIFY or DSBCAPS_CTRLALL - you wont' get a hardware
	// buffer
	dsbdSecondary.dwFlags = DSBCAPS_CTRL3D | DSBCAPS_STATIC;  
	dsbdSecondary.lpwfxFormat = &wfx;						 
	
	if( FAILED( hr = lpDS->CreateSoundBuffer( &dsbdSecondary, &lpSecondaryBuffer, NULL )))
	{
		printf("Failed to create dummy buffer");
		goto ID_ReleasePrimaryBuffer;
	}

	printf("Created dummy secondary buffer\n");
	
	if( FAILED( hr = lpSecondaryBuffer->QueryInterface( IID_IKsPropertySet,
						(void**)&lpPropertySet)))
	{
		printf("Failed to get Property Set Interface");
		goto ID_ReleaseSecondaryBuffer;
	}

	printf("Created Property Set Interface\n");

	// Do we have Voice Manager support ?
	if( FAILED( hr = lpPropertySet->QuerySupport(DSPROPSETID_VoiceManager,
			DSPROPERTY_VMANAGER_MODE, &ulSupport)))
	{
		printf("Failed to query support on Property Set Interface");
		goto ID_ReleasePropertySet;
	}
	else
	{
		// Check if we can get and set them.
		if ((ulSupport &(KSPROPERTY_SUPPORT_GET|KSPROPERTY_SUPPORT_SET)) != (KSPROPERTY_SUPPORT_GET|KSPROPERTY_SUPPORT_SET))
		{
			printf("Unable to SET and GET properties in the Voice Manager Property Set\n");
			goto ID_ReleasePropertySet;
		}
		else
		{
			printf("Voice Manager is present and working\n");
		}
	}

	// Switch on a Voice Manager Mode
	Mode = DSPROPERTY_VMANAGER_MODE_USER;
	lpPropertySet->Set(DSPROPSETID_VoiceManager,DSPROPERTY_VMANAGER_MODE,NULL,0,&Mode, 
				       sizeof(Mode));
	
	printf("Switched on USER mode\n");


	// Because our dummy buffer was created before we switched on the Voice Manager, we have
	// effectively lost one of our buffers. To re-claim the buffer, we need to release it AND
	// the property set interface created from it. (This is because when we created the PS
	// interface the reference count of the buffer was incremented - therefore the resources
	// are not released until both the DS3D Buffer and the Property Set are released)

	memset(&dsCaps, 0, sizeof(DSCAPS));
	dsCaps.dwSize = sizeof(DSCAPS);
	lpDS->GetCaps(&dsCaps);
	printf("With dummy buffer and Prop Set interfaces\n");
	printf("Available hw voices are %d\n", dsCaps.dwFreeHwMixingAllBuffers);
	
	lpSecondaryBuffer->Release();

	lpPropertySet->Release();

	memset(&dsCaps, 0, sizeof(DSCAPS));
	dsCaps.dwSize = sizeof(DSCAPS);
	lpDS->GetCaps(&dsCaps);
	printf("After releasing dummy buffer and Property Set interfaces\n");
	printf("Available hw voices are %d\n", dsCaps.dwFreeHwMixingAllBuffers);
	

	// We now have all of our soundcard's resources available, and because the Voice Manager
	// is still active, we can now create as many buffers as we like.

	// In a normal application we would now load in all our wavefiles, instead to keep 
	// things easier, we simply create another dummy buffer and duplicate it a few times
	// to demonstrate that we can make as many buffers as we need.

	// We will also create a Property Set interface for each buffer.  This is necessary if
	// you wish to use the Voice Manager USER mode (to set buffer priorities) or if you are
	// using EAX, and you would like control over the Reverb Mix for each individual buffer.

	memset(&dsbdSecondary, 0, sizeof(DSBUFFERDESC));
	memset(&wfx, 0, sizeof(WAVEFORMATEX));

	// Any format should do, e.g 11kHz 16 bit mono
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nChannels = 1;
	wfx.nSamplesPerSec = 11050;
	wfx.nAvgBytesPerSec = 22100;
	wfx.nBlockAlign = 2;
	wfx.wBitsPerSample = 16;
	wfx.cbSize = 0;

	dsbdSecondary.dwSize = sizeof(DSBUFFERDESC);
	dsbdSecondary.dwBufferBytes = 32;
	// Again, don't set DSBCAPS_CTRLPOSITIONNOTIFY or DSBCAPS_CTRLALL
	dsbdSecondary.dwFlags = DSBCAPS_CTRL3D | DSBCAPS_STATIC;   
	dsbdSecondary.lpwfxFormat = &wfx;
	
	if( FAILED( hr = lpDS->CreateSoundBuffer( &dsbdSecondary, &Buffers[0], NULL )))
	{
		printf("Failed to create dummy buffer ready for duplication");
		goto ID_ReleasePrimaryBuffer;
	}

	// Check to see that a hardware buffer was created
	memset(&dsbCaps, 0, sizeof(DSBCAPS));
	dsbCaps.dwSize = sizeof(DSBCAPS);

	Buffers[0]->GetCaps(&dsbCaps);

	if (!(dsbCaps.dwFlags & DSBCAPS_LOCHARDWARE))
	{
		printf("Buffer created in software\n");
		goto ID_ReleasePrimaryBuffer;
	}

	if( FAILED( hr = Buffers[0]->QueryInterface( IID_IKsPropertySet,
						(void**)&PropSet[0])))
	{
		printf("Failed to create Prop Set Interface for Buffer 0");
		Buffers[0]->Release();
		goto ID_ReleasePrimaryBuffer;
	}

	printf("Created new dummy buffer, ready for duplication\n");
	
	printf("1 created");  // our dummy buffer

	for(loop=1;loop<40;loop++)
	{
		if( FAILED(hr = lpDS->DuplicateSoundBuffer(Buffers[0], &Buffers[loop])))
		{
			printf(".  Failed to create buffer %d\n",loop+1);
			break;
		}
	
		// Check to see that a hardware buffer was created
		memset(&dsbCaps, 0, sizeof(DSBCAPS));
		dsbCaps.dwSize = sizeof(DSBCAPS);

		Buffers[loop]->GetCaps(&dsbCaps);

		if (!(dsbCaps.dwFlags & DSBCAPS_LOCHARDWARE))
		{
			printf("Buffer %d created in software\n",loop);
			goto ID_ReleaseAllBuffers;
		}
		
		if( FAILED( hr = Buffers[loop]->QueryInterface( IID_IKsPropertySet,
						(void**)&PropSet[loop])))
		{
			printf(".  Failed to get PS Interface for Buffer %d",loop+1);
			printf(", the buffer probably ended up in software.\n");
			goto ID_ReleaseAllBuffers;
		}

		printf(", %d created",loop+1);
	}

	printf("\n");

ID_ReleaseAllBuffers:
	for(loop=0;loop<40;loop++)
	{
		if (PropSet[loop])
		{
			PropSet[loop]->Release();
			PropSet[loop] = 0;
		}

		if (Buffers[loop])
		{
			Buffers[loop]->Release();
			Buffers[loop] = 0;
			printf("%d released, ",loop+1);
		}
	}
	goto ID_ReleasePrimaryBuffer;

ID_ReleasePropertySet:
	lpPropertySet->Release();
	printf("Released Property Set Interface\n");
ID_ReleaseSecondaryBuffer:
	lpSecondaryBuffer->Release();
	printf("Released dummy secondary buffer\n");
ID_ReleasePrimaryBuffer:
	lpdsbPrimary->Release();
	printf("\nReleased Primary buffer\n");
ID_ReleaseDirectSound:
	lpDS->Release();
	printf("Released Direct Sound Object\n");

ID_Exit:

	return 0;
}